import { AttributeType, gl } from '@antv/l7-core';
import { mat4 } from 'gl-matrix';
import BaseModel from '../../core/BaseModel';
import { HeatmapTriangulation } from '../../core/triangulation';
import { generateColorRamp } from '../../utils/color';
const heatmap3DFrag = "uniform sampler2D u_texture;\nuniform sampler2D u_colorTexture;\nuniform float u_opacity;\nvarying vec2 v_texCoord;\nvarying float v_intensity;\n\nvoid main(){\n   \n     float intensity = texture2D(u_texture, v_texCoord).r;\n     vec2 ramp_pos = vec2(\n        fract(16.0 * (1.0 - v_intensity)),\n        floor(16.0 * (1.0 - v_intensity)) / 16.0);\n    // vec4 color = texture2D(u_colorTexture,vec2(0.5,1.0-intensity));\n    vec4 color = texture2D(u_colorTexture,ramp_pos);\n    gl_FragColor = color;\n    gl_FragColor.a = color.a * smoothstep(0.1,0.2,intensity)* u_opacity;\n}\n";
const heatmap3DVert = "precision highp float;\nattribute vec3 a_Position;\nattribute vec2 a_Uv;\nuniform sampler2D u_texture;\nvarying vec2 v_texCoord;\nuniform mat4 u_ModelMatrix;\nuniform mat4 u_InverseViewProjectionMatrix;\nvarying float v_intensity;\n\nvec2 toBezier(float t, vec2 P0, vec2 P1, vec2 P2, vec2 P3) {\n    float t2 = t * t;\n    float one_minus_t = 1.0 - t;\n    float one_minus_t2 = one_minus_t * one_minus_t;\n    return (P0 * one_minus_t2 * one_minus_t + P1 * 3.0 * t * one_minus_t2 + P2 * 3.0 * t2 * one_minus_t + P3 * t2 * t);\n}\nvec2 toBezier(float t, vec4 p){\n    return toBezier(t, vec2(0.0, 0.0), vec2(p.x, p.y), vec2(p.z, p.w), vec2(1.0, 1.0));\n}\n#pragma include \"projection\"\nvoid main() {\n  v_texCoord = a_Uv;\n\n  vec2 pos = 1.8 * (a_Uv * vec2(2.0) - vec2(1.0));\n\n\n  vec4 p1 = vec4(pos, 0.0, 1.0);\n\tvec4 p2 = vec4(pos, 1.0, 1.0);\n\n\tvec4 inverseP1 = unproject_clipspace_to_position(p1, u_InverseViewProjectionMatrix);\n\tvec4 inverseP2 = unproject_clipspace_to_position(p2, u_InverseViewProjectionMatrix) ;\n\n  inverseP1 = inverseP1 / inverseP1.w;\n\tinverseP2 = inverseP2 / inverseP2.w;\n\n\tfloat zPos = (0.0 - inverseP1.z) / (inverseP2.z - inverseP1.z);\n\tvec4 position = inverseP1 + zPos * (inverseP2 - inverseP1);\n\n  vec4 b= vec4(0.5000, 0, 1, 0.5000);\n  float fh;\n\n  v_intensity = texture2D(u_texture, v_texCoord).r;\n  fh = toBezier(v_intensity, b).y;\n  gl_Position = project_common_position_to_clipspace(vec4(position.xy, fh * 50., 1.0));\n\n}\n";
const heatmapColorFrag = "uniform sampler2D u_texture;\nuniform sampler2D u_colorTexture;\nuniform float u_opacity;\nvarying vec2 v_texCoord;\nvarying float v_intensity;\n\nvoid main(){\n     float intensity = texture2D(u_texture, v_texCoord).r;\n     vec2 ramp_pos = vec2(\n        fract(16.0 * (1.0 - intensity)),\n        floor(16.0 * (1.0 - intensity)) / 16.0);\n    // vec4 color = texture2D(u_colorTexture,vec2(0.5,1.0-intensity));\n    vec4 color = texture2D(u_colorTexture,ramp_pos);\n    gl_FragColor = color;\n    gl_FragColor.a = color.a * smoothstep(0.,0.05,intensity) * u_opacity;\n\n}\n";
const heatmapFrag = "precision highp float;\nuniform float u_intensity;\nvarying float v_weight;\nvarying vec2 v_extrude;\n\n\nvoid main(){\n    float GAUSS_COEF = 0.3989422804014327;\n    float d = -0.5 * 3.0 * 3.0 * dot(v_extrude, v_extrude);\n    float val = v_weight * u_intensity * GAUSS_COEF * exp(d);\n    gl_FragColor = vec4(val, val, val, val);\n}\n";
const heatmapVert = "precision highp float;\nattribute vec3 a_Position;\nattribute float a_Size;\nattribute vec2 a_Dir;\nuniform float u_intensity;\nuniform float u_radius;\nvarying vec2 v_extrude;\nvarying float v_weight;\nuniform mat4 u_ModelMatrix;\n\n#pragma include \"projection\"\n\nvoid main(){\n    v_weight = a_Size;\n    float GAUSS_COEF = 0.3989422804014327;\n    float ZERO = 1.0 / 255.0 / 16.0;\n    float extrude_x = a_Dir.x * 2.0 -1.0;\n    float extrude_y = a_Dir.y * 2.0 -1.0;\n    vec2 extrude_dir = normalize(vec2(extrude_x,extrude_y));\n    float S = sqrt(-2.0 * log(ZERO / a_Size / u_intensity / GAUSS_COEF)) / 3.0;\n    v_extrude = extrude_dir * S;\n\n    vec2 offset = project_pixel(v_extrude * u_radius);\n    vec4 project_pos = project_position(vec4(a_Position.xy, 0.0, 1.0));\n\n    gl_Position = project_common_position_to_clipspace(vec4(project_pos.xy + offset, 0.0, 1.0));\n}\n";
const heatmapColorVert = "precision highp float;\nattribute vec3 a_Position;\nattribute vec2 a_Uv;\nuniform sampler2D u_texture;\nvarying vec2 v_texCoord;\nvarying float v_intensity;\nvoid main() {\n   v_texCoord = a_Uv;\n  v_intensity = texture2D(u_texture, v_texCoord).r;\n   gl_Position = vec4(a_Position.xy, 0, 1.);\n}\n";
import { heatMap3DTriangulation } from '../triangulation';
export default class HeatMapModel extends BaseModel {
  constructor(...args) {
    super(...args);
    this.texture = void 0;
    this.colorTexture = void 0;
    this.heatmapFramerBuffer = void 0;
    this.intensityModel = void 0;
    this.colorModel = void 0;
    this.shapeType = void 0;
  }

  render() {
    const {
      clear,
      useFramebuffer
    } = this.rendererService;
    useFramebuffer(this.heatmapFramerBuffer, () => {
      clear({
        color: [0, 0, 0, 0],
        depth: 1,
        stencil: 0,
        framebuffer: this.heatmapFramerBuffer
      });
      this.drawIntensityMode();
    });
    this.shapeType === 'heatmap' ? this.drawColorMode() : this.draw3DHeatMap();
  }

  getUninforms() {
    throw new Error('Method not implemented.');
  }

  buildModels() {
    var _shapeAttr$scale;

    const {
      createFramebuffer,
      clear,
      getViewportSize,
      createTexture2D,
      useFramebuffer
    } = this.rendererService;
    const shapeAttr = this.styleAttributeService.getLayerStyleAttribute('shape');
    const shapeType = (shapeAttr === null || shapeAttr === void 0 ? void 0 : (_shapeAttr$scale = shapeAttr.scale) === null || _shapeAttr$scale === void 0 ? void 0 : _shapeAttr$scale.field) || 'heatmap';
    this.shapeType = shapeType;
    this.intensityModel = this.buildHeatMapIntensity();
    this.colorModel = shapeType === 'heatmap' ? this.buildHeatmapColor() : this.build3dHeatMap();
    const {
      rampColors
    } = this.layer.getLayerConfig();
    const imageData = generateColorRamp(rampColors);
    const {
      width,
      height
    } = getViewportSize();
    this.heatmapFramerBuffer = createFramebuffer({
      color: createTexture2D({
        width,
        height,
        wrapS: gl.CLAMP_TO_EDGE,
        wrapT: gl.CLAMP_TO_EDGE,
        min: gl.LINEAR,
        mag: gl.LINEAR
      })
    });
    this.colorTexture = createTexture2D({
      data: imageData.data,
      width: imageData.width,
      height: imageData.height,
      wrapS: gl.CLAMP_TO_EDGE,
      wrapT: gl.CLAMP_TO_EDGE,
      min: gl.LINEAR,
      mag: gl.LINEAR,
      flipY: true
    });
    return [this.intensityModel, this.colorModel];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'dir',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Dir',
        buffer: {
          usage: gl.DYNAMIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3], vertex[4]];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: gl.DYNAMIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size = 2
          } = feature;
          return [size];
        }
      }
    });
  }

  buildHeatMapIntensity() {
    return this.layer.buildLayerModel({
      moduleName: 'heatmapintensity',
      vertexShader: heatmapVert,
      fragmentShader: heatmapFrag,
      triangulation: HeatmapTriangulation,
      depth: {
        enable: false
      },
      blend: {
        enable: true,
        func: {
          srcRGB: gl.ONE,
          srcAlpha: gl.ONE_MINUS_SRC_ALPHA,
          dstRGB: gl.ONE,
          dstAlpha: gl.ONE_MINUS_SRC_ALPHA
        }
      }
    });
  }

  buildHeatmapColor() {
    this.shaderModuleService.registerModule('heatmapColor', {
      vs: heatmapColorVert,
      fs: heatmapColorFrag
    });
    const {
      vs,
      fs,
      uniforms
    } = this.shaderModuleService.getModule('heatmapColor');
    const {
      createAttribute,
      createElements,
      createBuffer,
      createModel
    } = this.rendererService;
    return createModel({
      vs,
      fs,
      attributes: {
        a_Position: createAttribute({
          buffer: createBuffer({
            data: [-1, 1, 0, 1, 1, 0, -1, -1, 0, 1, -1, 0],
            type: gl.FLOAT
          }),
          size: 3
        }),
        a_Uv: createAttribute({
          buffer: createBuffer({
            data: [0, 1, 1, 1, 0, 0, 1, 0],
            type: gl.FLOAT
          }),
          size: 2
        })
      },
      uniforms: { ...uniforms
      },
      depth: {
        enable: false
      },
      blend: {
        enable: true,
        func: {
          srcRGB: gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      },
      count: 6,
      elements: createElements({
        data: [0, 2, 1, 2, 3, 1],
        type: gl.UNSIGNED_INT,
        count: 6
      })
    });
  }

  drawIntensityMode() {
    const {
      opacity,
      intensity = 10,
      radius = 5
    } = this.layer.getLayerConfig();
    this.intensityModel.draw({
      uniforms: {
        u_opacity: opacity || 1.0,
        u_radius: radius,
        u_intensity: intensity
      }
    });
  }

  drawColorMode() {
    const {
      opacity
    } = this.layer.getLayerConfig();
    this.colorModel.draw({
      uniforms: {
        u_opacity: opacity || 1.0,
        u_colorTexture: this.colorTexture,
        u_texture: this.heatmapFramerBuffer
      }
    });
  }

  draw3DHeatMap() {
    const {
      opacity
    } = this.layer.getLayerConfig();
    const invert = mat4.invert(mat4.create(), mat4.fromValues(...this.cameraService.getViewProjectionMatrix()));
    this.colorModel.draw({
      uniforms: {
        u_opacity: opacity || 1.0,
        u_colorTexture: this.colorTexture,
        u_texture: this.heatmapFramerBuffer,
        u_InverseViewProjectionMatrix: [...invert]
      }
    });
  }

  build3dHeatMap() {
    const {
      getViewportSize
    } = this.rendererService;
    const {
      width,
      height
    } = getViewportSize();
    const triangulation = heatMap3DTriangulation(width / 2.0, height / 2.0);
    this.shaderModuleService.registerModule('heatmap3dColor', {
      vs: heatmap3DVert,
      fs: heatmap3DFrag
    });
    const {
      vs,
      fs,
      uniforms
    } = this.shaderModuleService.getModule('heatmap3dColor');
    const {
      createAttribute,
      createElements,
      createBuffer,
      createModel
    } = this.rendererService;
    return createModel({
      vs,
      fs,
      attributes: {
        a_Position: createAttribute({
          buffer: createBuffer({
            data: triangulation.vertices,
            type: gl.FLOAT
          }),
          size: 3
        }),
        a_Uv: createAttribute({
          buffer: createBuffer({
            data: triangulation.uvs,
            type: gl.FLOAT
          }),
          size: 2
        })
      },
      primitive: gl.TRIANGLES,
      uniforms: { ...uniforms
      },
      depth: {
        enable: true
      },
      blend: {
        enable: true,
        func: {
          srcRGB: gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      },
      elements: createElements({
        data: triangulation.indices,
        type: gl.UNSIGNED_INT,
        count: triangulation.indices.length
      })
    });
  }

}
//# sourceMappingURL=heatmap.js.map