import BaseLayer from '../core/BaseLayer';
import LineModels from './models';
export default class LineLayer extends BaseLayer {
  constructor(...args) {
    super(...args);
    this.name = 'LineLayer';
    this.animateStartTime = 0;
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    this.models.forEach(model => model.draw({
      uniforms: this.layerModel.getUninforms()
    }));
    return this;
  }

  buildModels() {
    const shape = this.getModelType();
    this.layerModel = new LineModels[shape](this);
    this.models = this.layerModel.buildModels();
  }

  getModelType() {
    var _shapeAttribute$scale;

    const shapeAttribute = this.styleAttributeService.getLayerStyleAttribute('shape');
    const shape = shapeAttribute === null || shapeAttribute === void 0 ? void 0 : (_shapeAttribute$scale = shapeAttribute.scale) === null || _shapeAttribute$scale === void 0 ? void 0 : _shapeAttribute$scale.field;
    return shape || 'line';
  }

  initAnimate() {
    const {
      enable
    } = this.animateOptions;

    if (enable) {
      this.layerService.startAnimate();
      this.animateStartTime = this.layerService.clock.getElapsedTime();
    }
  }

}
//# sourceMappingURL=index.js.map