import _uniq from "lodash/uniq";
import _isString from "lodash/isString";
import _isNil from "lodash/isNil";

var _dec, _dec2, _dec3, _class, _class2, _descriptor, _descriptor2, _temp;

function _initializerDefineProperty(target, property, descriptor, context) { if (!descriptor) return; Object.defineProperty(target, property, { enumerable: descriptor.enumerable, configurable: descriptor.configurable, writable: descriptor.writable, value: descriptor.initializer ? descriptor.initializer.call(context) : void 0 }); }

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) { var desc = {}; Object.keys(descriptor).forEach(function (key) { desc[key] = descriptor[key]; }); desc.enumerable = !!desc.enumerable; desc.configurable = !!desc.configurable; if ('value' in desc || desc.initializer) { desc.writable = true; } desc = decorators.slice().reverse().reduce(function (desc, decorator) { return decorator(target, property, desc) || desc; }, desc); if (context && desc.initializer !== void 0) { desc.value = desc.initializer ? desc.initializer.call(context) : void 0; desc.initializer = undefined; } if (desc.initializer === void 0) { Object.defineProperty(target, property, desc); desc = null; } return desc; }

function _initializerWarningHelper(descriptor, context) { throw new Error('Decorating class property failed. Please ensure that ' + 'proposal-class-properties is enabled and runs after the decorators transform.'); }

import { ScaleTypes, StyleScaleType, TYPES } from '@antv/l7-core';
import { extent } from 'd3-array';
import * as d3 from 'd3-scale';
import { inject, injectable } from 'inversify';
const dateRegex = /^(?:(?!0000)[0-9]{4}([-/.]+)(?:(?:0?[1-9]|1[0-2])\1(?:0?[1-9]|1[0-9]|2[0-8])|(?:0?[13-9]|1[0-2])\1(?:29|30)|(?:0?[13578]|1[02])\1(?:31))|(?:[0-9]{2}(?:0[48]|[2468][048]|[13579][26])|(?:0[48]|[2468][048]|[13579][26])00)([-/.]?)0?2\2(?:29))(\s+([01]|([01][0-9]|2[0-3])):([0-9]|[0-5][0-9]):([0-9]|[0-5][0-9]))?$/;
const scaleMap = {
  [ScaleTypes.LINEAR]: d3.scaleLinear,
  [ScaleTypes.POWER]: d3.scalePow,
  [ScaleTypes.LOG]: d3.scaleLog,
  [ScaleTypes.IDENTITY]: d3.scaleIdentity,
  [ScaleTypes.TIME]: d3.scaleTime,
  [ScaleTypes.QUANTILE]: d3.scaleQuantile,
  [ScaleTypes.QUANTIZE]: d3.scaleQuantize,
  [ScaleTypes.THRESHOLD]: d3.scaleThreshold,
  [ScaleTypes.CAT]: d3.scaleOrdinal
};
let FeatureScalePlugin = (_dec = injectable(), _dec2 = inject(TYPES.IGlobalConfigService), _dec3 = inject(TYPES.ILogService), _dec(_class = (_class2 = (_temp = class FeatureScalePlugin {
  constructor() {
    _initializerDefineProperty(this, "configService", _descriptor, this);

    _initializerDefineProperty(this, "logger", _descriptor2, this);

    this.scaleCache = {};
    this.scaleOptions = {};
  }

  apply(layer, {
    styleAttributeService
  }) {
    layer.hooks.init.tap('FeatureScalePlugin', () => {
      this.scaleOptions = layer.getScaleOptions();
      const attributes = styleAttributeService.getLayerStyleAttributes();
      const {
        dataArray
      } = layer.getSource().data;
      this.caculateScalesForAttributes(attributes || [], dataArray);
    });
    layer.hooks.beforeRender.tap('FeatureScalePlugin', () => {
      this.scaleOptions = layer.getScaleOptions();
      const attributes = styleAttributeService.getLayerStyleAttributes();

      if (attributes) {
        const {
          dataArray
        } = layer.getSource().data;
        const attributesToRescale = attributes.filter(attribute => attribute.needRescale);

        if (attributesToRescale.length) {
          this.caculateScalesForAttributes(attributesToRescale, dataArray);
          this.logger.debug('rescale finished');
        }
      }
    });
  }

  isNumber(n) {
    return !isNaN(parseFloat(n)) && isFinite(n);
  }

  caculateScalesForAttributes(attributes, dataArray) {
    this.scaleCache = {};
    attributes.forEach(attribute => {
      if (attribute.scale) {
        const attributeScale = attribute.scale;
        const type = attribute.name;
        attributeScale.names = this.parseFields(attribute.scale.field || []);
        const scales = attributeScale.names.map(field => {
          return this.getOrCreateScale(field, attribute, dataArray);
        });

        if (scales.some(scale => scale.type === StyleScaleType.VARIABLE)) {
          attributeScale.type = StyleScaleType.VARIABLE;
          scales.forEach(scale => {
            if (!attributeScale.callback) {
              var _scale$option2;

              if (attributeScale.values) {
                var _scale$option;

                if (((_scale$option = scale.option) === null || _scale$option === void 0 ? void 0 : _scale$option.type) === 'linear' && attributeScale.values.length > 2) {
                  const tick = scale.scale.ticks(attributeScale.values.length);

                  if (type === 'color') {
                    scale.scale.domain(tick);
                  }
                }

                scale.scale.range(attributeScale.values);
              } else if (((_scale$option2 = scale.option) === null || _scale$option2 === void 0 ? void 0 : _scale$option2.type) === 'cat') {
                scale.scale.range(scale.option.domain);
              }
            }
          });
        } else {
          attributeScale.type = StyleScaleType.CONSTANT;
          attributeScale.values = scales.map((scale, index) => {
            return scale.scale(attributeScale.names[index]);
          });
        }

        attributeScale.scalers = scales.map(scale => {
          return {
            field: scale.field,
            func: scale.scale
          };
        });
        attribute.needRescale = false;
      }
    });
  }

  getOrCreateScale(field, attribute, dataArray) {
    const scalekey = [field, attribute.name].join('_');

    if (this.scaleCache[scalekey]) {
      return this.scaleCache[scalekey];
    }

    const styleScale = this.createScale(field, dataArray);
    this.scaleCache[scalekey] = styleScale;
    return this.scaleCache[scalekey];
  }

  parseFields(field) {
    if (Array.isArray(field)) {
      return field;
    }

    if (_isString(field)) {
      return field.split('*');
    }

    return [field];
  }

  createScale(field, data) {
    var _find;

    const scaleOption = this.scaleOptions[field];
    const styleScale = {
      field,
      scale: undefined,
      type: StyleScaleType.VARIABLE,
      option: scaleOption
    };

    if (!data || !data.length) {
      if (scaleOption && scaleOption.type) {
        styleScale.scale = this.createDefaultScale(scaleOption);
      } else {
        styleScale.scale = d3.scaleOrdinal([field]);
        styleScale.type = StyleScaleType.CONSTANT;
      }

      return styleScale;
    }

    const firstValue = (_find = data.find(d => !_isNil(d[field]))) === null || _find === void 0 ? void 0 : _find[field];

    if (this.isNumber(field) || _isNil(firstValue) && !scaleOption) {
      styleScale.scale = d3.scaleOrdinal([field]);
      styleScale.type = StyleScaleType.CONSTANT;
    } else {
      const type = scaleOption && scaleOption.type || this.getDefaultType(firstValue);
      const cfg = this.createDefaultScaleConfig(type, field, data);
      Object.assign(cfg, scaleOption);
      styleScale.scale = this.createDefaultScale(cfg);
      styleScale.option = cfg;
    }

    return styleScale;
  }

  getDefaultType(firstValue) {
    let type = ScaleTypes.LINEAR;

    if (typeof firstValue === 'string') {
      type = dateRegex.test(firstValue) ? ScaleTypes.TIME : ScaleTypes.CAT;
    }

    return type;
  }

  createDefaultScaleConfig(type, field, data) {
    const cfg = {
      type
    };
    const values = (data === null || data === void 0 ? void 0 : data.map(item => item[field])) || [];

    if (type !== ScaleTypes.CAT && type !== ScaleTypes.QUANTILE) {
      cfg.domain = extent(values);
    } else if (type === ScaleTypes.CAT) {
      cfg.domain = _uniq(values);
    } else if (type === ScaleTypes.QUANTILE) {
      cfg.domain = values;
    }

    return cfg;
  }

  createDefaultScale({
    type,
    domain
  }) {
    const scale = scaleMap[type]();

    if (domain) {
      scale.domain(domain);
    }

    return scale;
  }

}, _temp), (_descriptor = _applyDecoratedDescriptor(_class2.prototype, "configService", [_dec2], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
}), _descriptor2 = _applyDecoratedDescriptor(_class2.prototype, "logger", [_dec3], {
  configurable: true,
  enumerable: true,
  writable: true,
  initializer: null
})), _class2)) || _class);
export { FeatureScalePlugin as default };
//# sourceMappingURL=FeatureScalePlugin.js.map