var _dec, _class;

import { injectable } from 'inversify';
const lightTypeUniformMap = {
  directional: {
    lights: 'u_DirectionalLights',
    num: 'u_NumOfDirectionalLights'
  },
  spot: {
    lights: 'u_SpotLights',
    num: 'u_NumOfSpotLights'
  }
};
const DEFAULT_LIGHT = {
  type: 'directional',
  direction: [1, 10.5, 12],
  ambient: [0.2, 0.2, 0.2],
  diffuse: [0.6, 0.6, 0.6],
  specular: [0.1, 0.1, 0.1]
};
const DEFAULT_DIRECTIONAL_LIGHT = {
  direction: [0, 0, 0],
  ambient: [0, 0, 0],
  diffuse: [0, 0, 0],
  specular: [0, 0, 0]
};
const DEFAULT_SPOT_LIGHT = {
  position: [0, 0, 0],
  direction: [0, 0, 0],
  ambient: [0, 0, 0],
  diffuse: [0, 0, 0],
  specular: [0, 0, 0],
  constant: 1,
  linear: 0,
  quadratic: 0,
  angle: 14,
  exponent: 40,
  blur: 5
};
const COLOR_ATTRIBUTES = ['ambient', 'diffuse', 'specular'];
export function generateLightingUniforms(lights) {
  const lightsMap = {
    u_DirectionalLights: new Array(3).fill({ ...DEFAULT_DIRECTIONAL_LIGHT
    }),
    u_NumOfDirectionalLights: 0,
    u_SpotLights: new Array(3).fill({ ...DEFAULT_SPOT_LIGHT
    }),
    u_NumOfSpotLights: 0
  };

  if (!lights || !lights.length) {
    lights = [DEFAULT_LIGHT];
  }

  lights.forEach(({
    type = 'directional',
    ...rest
  }, i) => {
    const lightsUniformName = lightTypeUniformMap[type].lights;
    const lightsNumUniformName = lightTypeUniformMap[type].num;
    const num = lightsMap[lightsNumUniformName];
    lightsMap[lightsUniformName][num] = { ...lightsMap[lightsUniformName][num],
      ...rest
    };
    lightsMap[lightsNumUniformName]++;
  });
  return lightsMap;
}
let LightingPlugin = (_dec = injectable(), _dec(_class = class LightingPlugin {
  apply(layer) {
    layer.hooks.beforeRender.tap('LightingPlugin', () => {
      const {
        enableLighting
      } = layer.getLayerConfig();

      if (enableLighting) {
        layer.models.forEach(model => model.addUniforms({ ...generateLightingUniforms()
        }));
      }
    });
  }

}) || _class);
export { LightingPlugin as default };
//# sourceMappingURL=LightingPlugin.js.map