"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _BaseModel = _interopRequireDefault(require("../../core/BaseModel"));

var _triangulation = require("../../core/triangulation");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const heatmapGrid3dVert = "precision highp float;\n// \u591A\u8FB9\u5F62\u9876\u70B9\u5750\u6807\nattribute vec3 a_Position;\n// \u591A\u8FB9\u5F62\u7ECF\u7EAC\u5EA6\u5750\u6807\nattribute vec3 a_Pos;\n\nattribute vec3 a_Normal;\nattribute float a_Size;\nattribute vec4 a_Color;\nuniform vec2 u_radius;\nuniform float u_coverage: 0.9;\nuniform float u_angle: 0;\nuniform mat4 u_ModelMatrix;\nvarying vec4 v_color;\n\n#pragma include \"projection\"\n#pragma include \"project\"\n#pragma include \"light\"\n#pragma include \"picking\"\n\nvoid main() {\n  mat2 rotationMatrix = mat2(cos(u_angle), sin(u_angle), -sin(u_angle), cos(u_angle));\n  vec2 offset =(vec2(a_Position.xy * u_radius * rotationMatrix * u_coverage));\n  vec2 lnglat = unProjectFlat(a_Pos.xy + offset);\n  vec4 project_pos = project_position(vec4(lnglat, a_Position.z * a_Size, 1.0));\n  float lightWeight = calc_lighting(project_pos);\n  v_color =vec4(a_Color.rgb*lightWeight, a_Color.w);\n\n  gl_Position = project_common_position_to_clipspace(project_pos);\n  setPickingColor(a_PickingColor);\n}\n";
const heatmapGridFrag = "precision highp float;\nvarying vec4 v_color;\nuniform float u_opacity: 1;\n\n#pragma include \"picking\"\n\nvoid main() {\n  gl_FragColor = v_color;\n  gl_FragColor.a *= u_opacity;\n\n  gl_FragColor = filterColor(gl_FragColor);\n}\n";

class Grid3DModel extends _BaseModel.default {
  getUninforms() {
    const {
      opacity,
      coverage
    } = this.layer.getLayerConfig();
    return {
      u_opacity: opacity || 1.0,
      u_coverage: coverage || 1.0,
      u_radius: [this.layer.getSource().data.xOffset, this.layer.getSource().data.yOffset]
    };
  }

  buildModels() {
    return [this.layer.buildLayerModel({
      moduleName: 'grid3dheatmap',
      vertexShader: heatmapGrid3dVert,
      fragmentShader: heatmapGridFrag,
      triangulation: _triangulation.PointExtrudeTriangulation,
      depth: {
        enable: true
      }
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size
          } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'normal',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Normal',
        buffer: {
          usage: _l7Core.gl.STATIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'pos',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Pos',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx) => {
          const coordinates = feature.coordinates;
          return [coordinates[0], coordinates[1], 0];
        }
      }
    });
  }

}

exports.default = Grid3DModel;
//# sourceMappingURL=grid3d.js.map