"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _BaseModel = _interopRequireDefault(require("../../core/BaseModel"));

var _triangulation = require("../../core/triangulation");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const line_frag = "uniform float u_blur : 0.9;\nuniform float u_opacity : 1.0;\nuniform float u_dash_offset : 0.0;\nuniform float u_dash_ratio : 0.0;\nvarying vec4 v_color;\nvarying vec2 v_normal;\n\n#pragma include \"picking\"\n\nuniform float u_time;\nuniform vec4 u_aimate: [ 0, 2., 1.0, 0.2 ];\n// [animate, duration, interval, trailLength],\nvarying float v_distance_ratio;\nvarying float v_dash_array;\nvoid main() {\n  gl_FragColor = v_color;\n  // anti-alias\n  float blur = 1.- smoothstep(u_blur, 1., length(v_normal.xy)) * u_opacity;\n  // gl_FragColor.a *= blur;\n\n  #ifdef ANIMATE\n    float alpha =1.0 - fract( mod(1.0- v_distance_ratio,u_aimate.z)* (1.0/ u_aimate.z) + u_time / u_aimate.y);\n    alpha = (alpha + u_aimate.w -1.0) / u_aimate.w;\n    alpha = smoothstep(0., 1., alpha);\n    gl_FragColor.a *= alpha * blur;\n  #endif\n\n  gl_FragColor = filterColor(gl_FragColor);\n}\n";
const line_vert = "\nattribute float a_Miter;\nattribute vec4 a_Color;\nattribute vec2 a_Size;\nattribute vec3 a_Normal;\nattribute vec3 a_Position;\nuniform mat4 u_ModelMatrix;\n\n#pragma include \"projection\"\n#pragma include \"picking\"\n\nvarying vec4 v_color;\nvarying float v_dash_array;\nvarying vec2 v_normal;\n\nvoid main() {\n  v_normal = vec2(reverse_offset_normal(a_Normal) * sign(a_Miter));\n  v_color = a_Color;\n  vec3 size = a_Miter * a_Size.x * reverse_offset_normal(a_Normal);  //v_normal * vec3(1., -1., 1.0);\n  vec2 offset = project_pixel(size.xy);\n  vec4 project_pos = project_position(vec4(a_Position.xy, 0, 1.0));\n  gl_Position = project_common_position_to_clipspace(vec4(project_pos.xy + offset, a_Size.y, 1.0));\n\n  setPickingColor(a_PickingColor);\n}\n";

class LineModel extends _BaseModel.default {
  getUninforms() {
    const {
      opacity
    } = this.layer.getLayerConfig();
    return {
      u_opacity: opacity || 1.0
    };
  }

  buildModels() {
    return [this.layer.buildLayerModel({
      moduleName: 'line',
      vertexShader: line_vert,
      fragmentShader: line_frag,
      triangulation: _triangulation.LineTriangulation,
      blend: {
        enable: true,
        func: {
          srcRGB: _l7Core.gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: _l7Core.gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      }
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size = 1
          } = feature;
          return Array.isArray(size) ? [size[0], size[1]] : [size, 0];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'normal',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Normal',
        buffer: {
          usage: _l7Core.gl.STATIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'miter',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Miter',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[4]];
        }
      }
    });
  }

}

exports.default = LineModel;
//# sourceMappingURL=line.js.map