"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _inversify = require("inversify");

var _color = require("../utils/color");

var _dec, _class;

function encodePickingColor(featureIdx) {
  return [featureIdx + 1 & 255, featureIdx + 1 >> 8 & 255, featureIdx + 1 >> 8 >> 8 & 255];
}

const PickingStage = {
  NONE: 0.0,
  ENCODE: 1.0,
  HIGHLIGHT: 2.0
};
let PixelPickingPlugin = (_dec = (0, _inversify.injectable)(), _dec(_class = class PixelPickingPlugin {
  apply(layer, {
    rendererService,
    styleAttributeService
  }) {
    layer.hooks.init.tap('PixelPickingPlugin', () => {
      const {
        enablePicking
      } = layer.getLayerConfig();
      styleAttributeService.registerStyleAttribute({
        name: 'pickingColor',
        type: _l7Core.AttributeType.Attribute,
        descriptor: {
          name: 'a_PickingColor',
          buffer: {
            data: [],
            type: _l7Core.gl.FLOAT
          },
          size: 3,
          update: (feature, featureIdx) => enablePicking ? encodePickingColor(featureIdx) : [0, 0, 0]
        }
      });
    });
    layer.hooks.beforePickingEncode.tap('PixelPickingPlugin', () => {
      const {
        enablePicking
      } = layer.getLayerConfig();

      if (enablePicking) {
        layer.models.forEach(model => model.addUniforms({
          u_PickingStage: PickingStage.ENCODE
        }));
      }
    });
    layer.hooks.afterPickingEncode.tap('PixelPickingPlugin', () => {
      const {
        enablePicking
      } = layer.getLayerConfig();

      if (enablePicking) {
        layer.models.forEach(model => model.addUniforms({
          u_PickingStage: PickingStage.NONE,
          u_PickingColor: [0, 0, 0],
          u_HighlightColor: [0, 0, 0, 0]
        }));
      }
    });
    layer.hooks.beforeHighlight.tap('PixelPickingPlugin', pickedColor => {
      const {
        highlightColor
      } = layer.getLayerConfig();
      const highlightColorInArray = typeof highlightColor === 'string' ? (0, _color.rgb2arr)(highlightColor) : highlightColor || [1, 0, 0, 1];
      layer.models.forEach(model => model.addUniforms({
        u_PickingStage: PickingStage.HIGHLIGHT,
        u_PickingColor: pickedColor,
        u_HighlightColor: highlightColorInArray.map(c => c * 255)
      }));
    });
  }

}) || _class);
exports.default = PixelPickingPlugin;
//# sourceMappingURL=PixelPickingPlugin.js.map