"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _BaseModel = _interopRequireDefault(require("../../core/BaseModel"));

var _triangulation = require("../../core/triangulation");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const pointImageFrag = "\nuniform sampler2D u_texture;\nvarying vec4 v_color;\nvarying vec2 v_uv;\nuniform vec2 u_textSize;\nuniform float u_stroke_width : 1;\nuniform vec4 u_stroke_color : [1, 1, 1, 1];\nuniform float u_stroke_opacity : 1;\nuniform float u_opacity : 1;\n\nvarying float v_size;\nvoid main(){\nvec2 pos= v_uv / u_textSize + gl_PointCoord / u_textSize * 64.;\nvec2 fragmentPosition = 2.0*gl_PointCoord - 1.0;\nfloat distance = length(fragmentPosition);\nfloat distanceSqrd = distance * distance;\nfloat radius = 1.;\nfloat r = 1.0 - smoothstep(radius-(radius*0.01),\n                         radius+(radius*0.01),\n                         distanceSqrd);\n  vec4 textureColor=texture2D(u_texture,pos);\n  if(v_color == vec4(0.)){\n        gl_FragColor= vec4(textureColor.xyz, textureColor.w * r);\n  }else {\n        gl_FragColor= step(0.01, textureColor.x) * v_color;\n  }\n  return;\n}\n";
const pointImageVert = "\nprecision highp float;\nattribute vec3 a_Position;\nattribute vec4 a_Color;\nattribute vec2 a_Uv;\nattribute float a_Size;\nvarying vec4 v_color;\nvarying vec2 v_uv;\nuniform mat4 u_ModelMatrix;\nuniform float u_stroke_width : 1;\nvarying float v_size;\n\n#pragma include \"projection\"\n\nvoid main() {\n   v_color = a_Color;\n   v_uv = a_Uv;\n   vec4 project_pos = project_position(vec4(a_Position, 1.0));\n   v_size = a_Size;\n\n   gl_Position = project_common_position_to_clipspace(vec4(project_pos.xyz, 1.0));\n   gl_PointSize = a_Size * 2.0 * u_DevicePixelRatio;\n\n}\n";

class ImageModel extends _BaseModel.default {
  constructor(...args) {
    super(...args);
    this.texture = void 0;
  }

  getUninforms() {
    const {
      opacity
    } = this.layer.getLayerConfig();
    return {
      u_opacity: opacity || 1.0,
      u_texture: this.texture,
      u_textSize: [1024, this.iconService.canvasHeight || 128]
    };
  }

  buildModels() {
    this.registerBuiltinAttributes();
    this.updateTexture();
    this.iconService.on('imageUpdate', () => {
      this.updateTexture();
      this.layer.render();
    });
    return [this.layer.buildLayerModel({
      moduleName: 'pointImage',
      vertexShader: pointImageVert,
      fragmentShader: pointImageFrag,
      triangulation: _triangulation.PointImageTriangulation,
      primitive: _l7Core.gl.POINTS,
      depth: {
        enable: false
      },
      blend: {
        enable: true,
        func: {
          srcRGB: _l7Core.gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: _l7Core.gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      }
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size
          } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'uv',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Uv',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const iconMap = this.iconService.getIconMap();
          const {
            shape
          } = feature;
          const {
            x,
            y
          } = iconMap[shape] || {
            x: 0,
            y: 0
          };
          return [x, y];
        }
      }
    });
  }

  updateTexture() {
    const {
      createTexture2D
    } = this.rendererService;
    this.texture = createTexture2D({
      data: this.iconService.getCanvas(),
      width: 1024,
      height: this.iconService.canvasHeight || 128
    });
  }

}

exports.default = ImageModel;
//# sourceMappingURL=image.js.map