"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PointTriangulation = PointTriangulation;
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _BaseModel = _interopRequireDefault(require("../../core/BaseModel"));

var _color = require("../../utils/color");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const normalFrag = "\nuniform float u_opacity : 1;\nvarying vec4 v_color;\n\n#pragma include \"picking\"\n\nvoid main() {\n  gl_FragColor = v_color;\n  gl_FragColor.a = u_opacity;\n\n  gl_FragColor = filterColor(gl_FragColor);\n}\n";
const normalVert = "\nattribute vec3 a_Position;\nuniform mat4 u_ModelMatrix;\nattribute float a_Size;\nattribute vec4 a_Color;\nvarying vec4 v_color;\n\n#pragma include \"projection\"\n#pragma include \"picking\"\n\nvoid main() {\n  v_color = a_Color;\n  vec4 project_pos = project_position(vec4(a_Position, 1.0));\n  gl_Position = project_common_position_to_clipspace(project_pos);\n  gl_PointSize = a_Size * 2.0 * u_DevicePixelRatio;\n  setPickingColor(a_PickingColor);\n}\n";

function PointTriangulation(feature) {
  const coordinates = feature.coordinates;
  return {
    vertices: [...coordinates],
    indices: [0],
    size: coordinates.length
  };
}

class NormalModel extends _BaseModel.default {
  getUninforms() {
    const {
      opacity = 1,
      strokeColor = 'rgb(0,0,0,0)',
      strokeWidth = 1
    } = this.layer.getLayerConfig();
    return {
      u_opacity: opacity,
      u_stroke_width: strokeWidth,
      u_stroke_color: (0, _color.rgb2arr)(strokeColor)
    };
  }

  buildModels() {
    return [this.layer.buildLayerModel({
      moduleName: 'normalpoint',
      vertexShader: normalVert,
      fragmentShader: normalFrag,
      triangulation: PointTriangulation,
      depth: {
        enable: false
      },
      primitive: _l7Core.gl.POINTS,
      blend: {
        enable: true,
        func: {
          srcRGB: _l7Core.gl.ONE,
          srcAlpha: 1,
          dstRGB: _l7Core.gl.ONE,
          dstAlpha: 1
        }
      }
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size
          } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
  }

}

exports.default = NormalModel;
//# sourceMappingURL=normal.js.map