"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _BaseModel = _interopRequireDefault(require("../../core/BaseModel"));

var _triangulation = require("../../core/triangulation");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const polygon_frag = "uniform float u_opacity: 1.0;\nvarying vec4 v_Color;\n\n#pragma include \"picking\"\n\nvoid main() {\n  gl_FragColor = v_Color;\n  gl_FragColor.a *= u_opacity;\n  gl_FragColor = filterColor(gl_FragColor);\n}\n";
const polygon_vert = "attribute vec4 a_Color;\nattribute vec3 a_Position;\nattribute vec3 a_Normal;\nattribute float a_Size;\nuniform mat4 u_ModelMatrix;\n\nvarying vec4 v_Color;\n\n#pragma include \"projection\"\n#pragma include \"picking\"\n\nvoid main() {\n  v_Color = a_Color;\n  vec4 project_pos = project_position(vec4(a_Position, 1.0));\n  gl_Position = project_common_position_to_clipspace(vec4(project_pos.xyz, 1.0));\n\n  setPickingColor(a_PickingColor);\n}\n\n";

class FillModel extends _BaseModel.default {
  getUninforms() {
    const {
      opacity = 1
    } = this.layer.getLayerConfig();
    return {
      u_opacity: opacity
    };
  }

  buildModels() {
    return [this.layer.buildLayerModel({
      moduleName: 'polygon',
      vertexShader: polygon_vert,
      fragmentShader: polygon_frag,
      triangulation: _triangulation.polygonTriangulation,
      depth: {
        enable: false
      }
    })];
  }

  registerBuiltinAttributes() {}

}

exports.default = FillModel;
//# sourceMappingURL=fill.js.map