"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _BaseLayer = _interopRequireDefault(require("../core/BaseLayer"));

var _triangulation = require("../core/triangulation");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const rasterImageFrag = "precision mediump float;\nuniform float u_opacity: 1.0;\nuniform sampler2D u_texture;\nvarying vec2 v_texCoord;\nvoid main() {\n  vec4 color = texture2D(u_texture,vec2(v_texCoord.x,v_texCoord.y));\n  gl_FragColor = color;\n}\n";
const rasterImageVert = "precision highp float;\nuniform mat4 u_ModelMatrix;\nattribute vec3 a_Position;\nattribute vec2 a_Uv;\nvarying vec2 v_texCoord;\n#pragma include \"projection\"\nvoid main() {\n   v_texCoord = a_Uv;\n   vec4 project_pos = project_position(vec4(a_Position, 1.0));\n   gl_Position = project_common_position_to_clipspace(vec4(project_pos.xy,0., 1.0));\n}\n";

class ImageLayer extends _BaseLayer.default {
  constructor(...args) {
    super(...args);
    this.name = 'ImageLayer';
    this.texture = void 0;
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    const {
      opacity
    } = this.getLayerConfig();

    if (this.texture) {
      this.models.forEach(model => model.draw({
        uniforms: {
          u_opacity: opacity || 1,
          u_texture: this.texture
        }
      }));
    }

    return this;
  }

  buildModels() {
    this.registerBuiltinAttributes();
    const source = this.getSource();
    const {
      createTexture2D
    } = this.rendererService;
    source.data.images.then(imageData => {
      this.texture = createTexture2D({
        data: imageData[0],
        width: imageData[0].width,
        height: imageData[0].height
      });
      this.renderModels();
    });
    this.models = [this.buildLayerModel({
      moduleName: 'RasterImage',
      vertexShader: rasterImageVert,
      fragmentShader: rasterImageFrag,
      triangulation: _triangulation.RasterImageTriangulation,
      primitive: _l7Core.gl.TRIANGLES,
      depth: {
        enable: false
      },
      blend: {
        enable: true,
        func: {
          srcRGB: _l7Core.gl.SRC_ALPHA,
          srcAlpha: 1,
          dstRGB: _l7Core.gl.ONE_MINUS_SRC_ALPHA,
          dstAlpha: 1
        }
      }
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'uv',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Uv',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 2,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          return [vertex[3], vertex[4]];
        }
      }
    });
  }

}

exports.default = ImageLayer;
//# sourceMappingURL=image.js.map